#
#     This R script is part of the bundle used in the paper:
#
#     Modeling tissue contamination to improve molecular identification
#     of the primary tumor site of metastases. 	
#
#     Description of this R script:
#     Scripts for fitting ANOVA+PAM predictors
#
#     Intended for use with R.
#     Copyright (C) 2013 Martin Vincent
# 
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>
#

library(pamr)

fold.change <- function(x, classes) {
fc <- sapply(1:ncol(x), function(i) {
tmp <- sapply(levels(classes), function(cls) mean(x[classes == cls,i])) 
#return(max(tmp)-min(tmp))
return(max(abs(mean(tmp)-tmp)))
})
return(fc)
}


all.elements.equal <- function(v) {
return(all(v == v[1]))
}

anova.pam.fit <- function(x, classes, xnew, weights = rep(1, length(classes)), fc.cutoff = .6, max.features = 100) {

## weights -> prior

# check that all weights within classes are equal
if(!all(sapply(levels(classes), function(cls) all.elements.equal(weights[classes == cls])))) {
stop("weights unequal within classes")
}

prior <- sapply(levels(classes), function(cls) sum(weights[classes == cls]))/sum(weights)

## ANOVA
#pvalues <- sapply(1:ncol(x), function(i) anova(lm(x[,i] ~ classes, weights = weights))$"Pr(>F)"[1])
pvalues <- sapply(1:ncol(x), function(i) anova(lm(x[,i] ~ classes))$"Pr(>F)"[1])

fc <- fold.change(x, classes)

# Reorder
fc <- fc[order(pvalues)]
x <- x[,order(pvalues)]
xnew <- xnew[,order(pvalues),drop=FALSE]

# fold changes filter
x <- x[,fc >= fc.cutoff]
xnew <- xnew[,fc >= fc.cutoff,drop=FALSE]

# max features
x <- x[,1:max.features]
xnew <- xnew[,1:max.features,drop=FALSE]

res <- list()
res$classes <- matrix(nrow = nrow(xnew), ncol = ncol(x)-9)
mode(res$classes) <- "character"

for(k in 10:ncol(x)) {

pam.data <- list()
pam.data$x <- t(x[,1:k])
pam.data$y <- classes

fit <- pamr.train(pam.data, prior = prior)

res$classes[,k-9] <- as.character(pamr.predict(fit, t(xnew[,1:k,drop=FALSE]), threshold = 0))

}

res$features <- 10:ncol(x)

return(res)

}
