#	
#     This R script is part of the bundle used in the paper:
#
#     Modeling tissue contamination to improve molecular identification
#     of the primary tumor site of metastases. 	
#
#     Description of this R script:
#     Produce a plot similar to Figure 3 in the paper.
#
#     Intended for use with R.
#     Copyright (C) 2013 Martin Vincent
# 
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>
#

library(ggplot2)
library(reshape2)
library(plyr)

argmax <- function(x) which(max(x) == x)
argmin <- function(x) which(min(x) == x)

theme_set(theme_bw())

res <- results
res$features <- round(res$features)
res <- ddply(res, .variables = c("class", "classifier", "features", "n"), .fun = function(x) x[argmin(x$err)[1],])

levels(res$class) <- c("all", "Metastatic liver core biopsies", "Liver related classes", "Low tumor content", "High tumor content")
levels(res$classifier) <- c("Primary tumors", "Artificial (log scale)", "Artificial (linear scale)")

res <- subset(res, classifier != "CB")
res$classifier <- factor(res$classifier)

res$err <- res$err*100 #to percentage

fig3 <- ggplot(subset(res, class == "all" & n == 0), aes(x=features, y=err)) +
		geom_line(aes(linetype=factor(classifier))) +
		scale_y_continuous(name="Misclassification percentage",  limit = c(30, 70)) + 
		scale_x_continuous(name="miRNAs included in the model", limit=c(20, 130)) + 
		theme(legend.position = "bottom", legend.direction='horizontal', legend.key = element_blank()) +
		labs(linetype = "Training samples") 

print(fig3)
