#	
#     This R script is part of the bundle used in the paper:
#
#     Modeling tissue contamination to improve molecular identification
#     of the primary tumor site of metastases. 	
#
#     Description of this R script:
#     Leave-one-out cross-validation of predictor based on metastatic tumor samples
#
#     Intended for use with R.
#     Copyright (C) 2013 Martin Vincent
# 
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>
#

library(msgl)
source('load_data.R')

lambda <- msgl.lambda.seq(x.cb, classes.cb, alpha = 0, d = build$d, standardize = FALSE, lambda.min = 1e-5)
fit.cv <- msgl.cv(x.cb, classes.cb, lambda = lambda, alpha = 0, cv.indices = lapply(1:75, function(x) x), standardize = FALSE, max.threads = 8L)

err <- colSums(fit.cv$classes != classes.cb)/length(classes.cb)

# Plot error as a function features include in the model
plot(x = fit.cv$features, y = err, type = "l")

