/*
 * rObject.h
 *
 *  Created on: Jul 30, 2011
 *      Author: martin
 */

#ifndef ROBJECT_H_
#define ROBJECT_H_

class rObject {

private:

	R::SEXP exp;

	arma::u32 number_of_protects;

	bool const unprotect_on_destruction;

public:

	rObject(arma::u32 value, bool unprotect_on_destruction = true) :
			number_of_protects(1), unprotect_on_destruction(
					unprotect_on_destruction) {
		R::PROTECT(exp = R::allocVector(INTSXP, 1));
		R::INTEGER(exp)[0] = value;
	}

	rObject(double value, bool unprotect_on_destruction = true) :
			number_of_protects(1), unprotect_on_destruction(
					unprotect_on_destruction) {
		R::PROTECT(exp = R::allocVector(REALSXP, 1));
		R::REAL(exp)[0] = value;
	}

	rObject(arma::Mat<double> const& m, bool unprotect_on_destruction = true) :
			number_of_protects(2), unprotect_on_destruction(
					unprotect_on_destruction) {

		R::SEXP matrixDim;
		R::PROTECT(matrixDim = R::allocVector(INTSXP, 2));
		R::INTEGER(matrixDim)[0] = m.n_rows;
		R::INTEGER(matrixDim)[1] = m.n_cols;

		R::PROTECT(exp = R::allocVector(REALSXP, m.n_rows * m.n_cols));

		//Copy data
		arma::mat t(R::REAL(exp), m.n_rows, m.n_cols, false, true);
		t = conv_to < arma::mat > ::from(m);

		setAttrib(exp, R::R_DimSymbol, matrixDim);
	}

	rObject(arma::Mat<arma::u32> const& m, bool unprotect_on_destruction = true) :
			number_of_protects(2), unprotect_on_destruction(
					unprotect_on_destruction) {

		R::SEXP matrixDim;
		R::PROTECT(matrixDim = R::allocVector(INTSXP, 2));
		R::INTEGER(matrixDim)[0] = m.n_rows;
		R::INTEGER(matrixDim)[1] = m.n_cols;

		R::PROTECT(exp = R::allocVector(INTSXP, m.n_rows * m.n_cols));

		//Copy data
		Mat<int> t(R::INTEGER(exp), m.n_rows, m.n_cols, false, true);
		t = conv_to < arma::Mat<int> > ::from(m);

		setAttrib(exp, R::R_DimSymbol, matrixDim);
	}

	rObject(Col<double> const& v, bool unprotect_on_destruction = true) :
			number_of_protects(1), unprotect_on_destruction(
					unprotect_on_destruction) {

		R::PROTECT(exp = R::allocVector(REALSXP, v.n_elem));

		//Copy data
		vec t(R::REAL(exp), v.n_elem, false, true);
		t = conv_to < arma::vec > ::from(v);
	}

	rObject(Col<arma::u32> const& v, bool unprotect_on_destruction = true) :
			number_of_protects(1), unprotect_on_destruction(
					unprotect_on_destruction) {

		R::PROTECT(exp = R::allocVector(INTSXP, v.n_elem));

		//Copy data
		arma::Col<int> t(R::INTEGER(exp), v.n_elem, false, true);
		t = conv_to < arma::Col<int> > ::from(v);
	}

	rObject(Indices i, bool unprotect_on_destruction = true) :
			number_of_protects(1), unprotect_on_destruction(
					unprotect_on_destruction) {

		R::PROTECT(exp = R::allocVector(INTSXP, i.size()));

		//Copy data
		arma::Col<int> t(R::INTEGER(exp), i.size(), false, true);
		t = conv_to < arma::Col<int> > ::from(i.getElements());
	}

	template<typename T>
	rObject(arma::field<T> const& field, bool unprotect_on_destruction = true) :
			number_of_protects(1), unprotect_on_destruction(
					unprotect_on_destruction) {

		R::PROTECT(exp = R::allocVector(VECSXP, field.n_elem)); // Creating a list with n_elem elements

		//Construct list
		unsigned int i;
		for (i = 0; i < field.n_elem; i++) {
			// attaching
			rObject tmp(field(i), false);
			number_of_protects += tmp.n_protects();
			R::SET_VECTOR_ELT(exp, i, tmp);
		}

	}

	~rObject() {
		if (unprotect_on_destruction) {
			R::UNPROTECT(number_of_protects);
		}
	}

	operator R::SEXP() const {
		return getSEXP();
	}

	R::SEXP getSEXP() const {
		return exp;
	}

	arma::u32 n_protects() const {
		return number_of_protects;
	}

};

#endif /* ROBJECT_H_ */
