/*
 * subsample.cpp
 *
 *  Created on: Dec 11, 2011
 *      Author: martin
 */

//Debugging
#define ARMA_NO_DEBUG
#define SGL_DIM_CHECKS

//Exception handling
#define SGL_EXCEPTION_AS_ERROR

// Map messages, errors and warnings
void show_warning(const char * msg);
void report_error(const char *msg);
void show_msg(const char * msg);
#define SGL_WARNING(msg) show_warning(msg);
#define SGL_ERROR(msg) report_error(msg);
#define SGL_MSG(msg) show_msg(msg);

#include <string>
#include <armadillo.hpp>

std::string create_error_msg(const char * msg, const char * file_name, int line_number) {

	std::ostringstream error_msg;

	error_msg << msg << " ( in " << file_name << " at line " << line_number << " )";

	return error_msg.str();
}

#define ASSERT(condition, msg) if(!(condition)) throw std::runtime_error(create_error_msg(msg, __FILE__, __LINE__));

#include <Indices/GroupedIndices.h>
#include <numeric.h>
#include <rtools/rtools.h>

#include <memory>

template<typename R, typename T>
arma::field<R> conv(arma::field<T> source) {

	arma::field<R> target(source.n_elem);

	for (sgl::natural i = 0; i < source.n_elem; ++i) {
		target(i) = source(i);
	}

	return target;
}

#ifdef __cplusplus
extern "C" {
#endif

R::SEXP cvsamples(R::SEXP r_classes, R::SEXP r_fold, R::SEXP r_seed);

R::SEXP subsamples_fraction(R::SEXP r_classes, R::SEXP r_numer_of_subsamples,
		R::SEXP r_subsample_size_fraction, R::SEXP r_seed);

R::SEXP subsamples_count(R::SEXP r_classes, R::SEXP r_numer_of_subsamples,
		R::SEXP r_count, R::SEXP r_seed);

#ifdef __cplusplus
}
#endif

void show_warning(const char * msg) {
	R::Rf_warning(msg);
}

void show_msg(const char * msg) {
	R::Rprintf(msg);
}

void report_error(const char *msg) {
	R::Rf_error(msg);
}

R::SEXP cvsamples(R::SEXP r_classes, R::SEXP r_fold, R::SEXP r_seed) {

	//Map data
	const sgl::natural_vector Y = get_value < sgl::natural_vector > (r_classes);
	const sgl::natural fold = get_value < sgl::natural > (r_fold);
	const unsigned int seed = get_value<unsigned int>(r_seed);

	field < Indices > cvgroups;

	GroupedIndices const indices(0, Y.n_elem - 1, Y);

	boost::mt19937 gen;
	gen.seed(seed);

	cvgroups = conv < Indices, GroupedIndices
			> (indices.groupedDisjointSubsets(fold, gen));

	return rObject(cvgroups);
}

R::SEXP subsamples_fraction(R::SEXP r_classes, R::SEXP r_numer_of_subsamples,
		R::SEXP r_subsample_size_fraction, R::SEXP r_seed) {

	const sgl::natural_vector Y = get_value < sgl::natural_vector > (r_classes);
	const sgl::natural numer_of_subsamples = get_value < sgl::natural
			> (r_numer_of_subsamples);
	const sgl::numeric subsample_size_fraction = get_value < sgl::numeric
			> (r_subsample_size_fraction);
	const unsigned int seed = get_value<unsigned int>(r_seed);

	GroupedIndices const indices(0, Y.n_elem - 1, Y);

	field < Indices > subsamples;

	boost::mt19937 gen;
	gen.seed(seed);

	subsamples = conv < Indices, GroupedIndices
			> (indices.blancedRandomSubsets(subsample_size_fraction,
					numer_of_subsamples, gen));

	return rObject(subsamples);
}

R::SEXP subsamples_count(R::SEXP r_classes, R::SEXP r_numer_of_subsamples,
		R::SEXP r_count, R::SEXP r_seed) {

	const sgl::natural_vector Y = get_value < sgl::natural_vector > (r_classes);
	const sgl::natural numer_of_subsamples = get_value < sgl::natural
			> (r_numer_of_subsamples);
	const sgl::natural count = get_value < sgl::natural
			> (r_count);
	const unsigned int seed = get_value<unsigned int>(r_seed);

	GroupedIndices const indices(0, Y.n_elem - 1, Y);

	field < Indices > subsamples;

	boost::mt19937 gen;
	gen.seed(seed);

	subsamples = conv < Indices, GroupedIndices
			> (indices.blancedRandomSubsets(count, numer_of_subsamples, gen));

	return rObject(subsamples);
}
