#	
#     This R script is part of the bundle used in the paper:
#
#     Modeling tissue contamination to improve molecular identification
#     of the primary tumor site of metastases. 	
#
#     Description of this R script:
#     Validation of ANOVA+PAM lasso predictor, 
#     based on a combination of the resections and core biopsies
#
#     Intended for use with R.
#     Copyright (C) 2013 Martin Vincent
# 
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>
#

library(subsamples)

#TODO update file path
source('anova+pam_fit.R')
source('load_data.R')

results <- NULL

cls.list <- list()
cls.list$non.liver <- c("Breast", "CRC", "EG", "Pancreas",  "Squamous")
cls.list$liver <- c("Liver", "Cirrhosis", "CCA", "HCC")

### Sub samples

number.of.subsamples <- 100L 

subs.test <- sub.samples(classes.cb, d=number.of.subsamples, fraction = .5)

standardize <- FALSE

tp.cb <- cut(as.numeric(sub("%", "", data.cb$tumor))/100, breaks = c(0,1/2, 1))

###########################
###
### PRIM + CB
###

print("PRIM + CB")

for(n in c(2L, 4L)) {

## Traning samples
if(n == 0) {
  subs.train <- lapply(subs.test, function(s) c())
} else {
  subs.train <- lapply(subs.test, function(s) which(names(classes.cb) %in% names(classes.cb[-s])[classes.cb[-s] %in% cls.list$non.liver][sub.samples(classes.cb[-s][classes.cb[-s] %in% cls.list$non.liver], d=1L, n = n)[[1]]]))
  #subs.train <- lapply(subs.test, function(s) which(names(classes.cb) %in% names(classes.cb[-s])[sub.samples(classes.cb[-s], d=1L, n = n)[[1]]]))
}

### Check that there is no bug in the code for sampleing test and traning sets 
lapply(1:length(subs.test), function(i) if(any(subs.test[[i]] %in% subs.train[[i]])) stop("Error - test samples found in training samples"))

## Init
err <- vector("numeric", 91)
err.cls <- list()

for(i in 1:length(cls.list)) {
  err.cls[[i]] <- vector("numeric", 91)
}

err.tp <- matrix(0, nrow = 91, ncol = 2)

for(i in 1:length(subs.test)) {

x.comp <- rbind(x.prim, x.cb[subs.train[[i]],,drop = FALSE])
classes.comp <- factor(c(as.character(classes.prim), as.character(classes.cb)[subs.train[[i]]]))

res.cb <- anova.pam.fit(x.comp, classes.comp, x.cb[subs.test[[i]],,drop = FALSE])

## Err
err <- err + colSums(res.cb$classes != classes.cb[subs.test[[i]]])/length(subs.test[[i]])

for(j in 1:length(cls.list)) {
sel <- classes.cb[subs.test[[i]]] %in% cls.list[[j]]
err.cls[[j]] <- err.cls[[j]] + colSums(res.cb$classes[sel,] != classes.cb[subs.test[[i]]][sel])/sum(sel)
}

## Tp errors
sel <- classes.cb[subs.test[[i]]] %in% cls.list$non.liver
errors.matrix <- res.cb$classes[sel,, drop = FALSE ] != classes.cb[subs.test[[i]]][sel]
err.tp <- err.tp + sapply(levels(tp.cb), function(q) colMeans(errors.matrix[tp.cb[subs.test[[i]]][sel] == q,,drop=FALSE ]))

}

### 
features <- res.cb$features

err <- err/length(subs.test)
results <- rbind(results, data.frame(class = "all", err = err, features = features, n=n, classifier = "PRIM"))

for(j in 1:length(cls.list)) {
  err.cls[[j]] <- err.cls[[j]]/length(subs.test)
  results <- rbind(results, data.frame(class = names(cls.list)[j], err = err.cls[[j]], features = features, n= n, classifier = "PRIM"))
}

err.tp <- err.tp/length(subs.test)
results <- rbind(results, data.frame(class = "low", err = err.tp[,1], features = features, n= n, classifier = "PRIM"))
results <- rbind(results, data.frame(class = "high", err = err.tp[,2], features = features, n= n, classifier = "PRIM"))

}
